"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processStartUpHooks = exports.validateMinimalConfig = void 0;
const dispatcher_1 = require("../../dispatcher");
const logger_1 = require("../../../logs/logger");
const checks_1 = require("../../checks");
const scm_1 = require("../../scm");
const configHelpers_1 = require("../../config/configHelpers");
const validateMinimalConfig = async (clientOpts) => {
    if (!clientOpts.config.brokerToken &&
        !clientOpts.config.universalBrokerEnabled) {
        const brokerToken = clientOpts.config.brokerToken;
        // null, undefined, empty, etc.
        logger_1.log.error({ brokerToken }, '[MISSING_BROKER_TOKEN] BROKER_TOKEN is required to successfully identify itself to the server');
        const error = new ReferenceError('BROKER_TOKEN is required to successfully identify itself to the server');
        error['code'] = 'MISSING_BROKER_TOKEN';
        throw error;
    }
    if (!clientOpts.config.brokerServerUrl) {
        const brokerServerUrl = clientOpts.config.brokerServerUrl;
        // null, undefined, empty, etc.
        logger_1.log.error({ brokerServerUrl }, '[MISSING_BROKER_SERVER_URL] BROKER_SERVER_URL is required to connect to the broker server');
        const error = new ReferenceError('BROKER_SERVER_URL is required to connect to the broker server');
        error['code'] = 'MISSING_BROKER_SERVER_URL';
        throw error;
    }
};
exports.validateMinimalConfig = validateMinimalConfig;
const processStartUpHooks = async (clientOpts, brokerClientId) => {
    var _a, _b;
    try {
        // if (!clientOpts.config.BROKER_CLIENT_URL) {
        //   const proto =
        //     !clientOpts.config.key && !clientOpts.config.cert ? 'http' : 'https';
        //   clientOpts.config.BROKER_CLIENT_URL = `${proto}://localhost:${clientOpts.port}`;
        // }
        let preflightCheckResults = [];
        if ((0, checks_1.preflightChecksEnabled)(clientOpts.config)) {
            // wrap preflight checks execution into try-catch, so the broker client
            // will start anyway
            try {
                preflightCheckResults = await (0, checks_1.executePreflightChecks)(clientOpts.config);
            }
            catch (error) {
                logger_1.log.error({ error }, 'failed to execute preflight checks');
            }
        }
        let serverId;
        if ((0, configHelpers_1.highAvailabilityModeEnabled)(clientOpts.config)) {
            if (clientOpts.config.universalBrokerEnabled) {
                for (const key in clientOpts.config.connections) {
                    serverId = await (0, dispatcher_1.getServerId)(clientOpts.config, clientOpts.config.connections[key].identifier, brokerClientId);
                    if (serverId === null) {
                        logger_1.log.warn({}, 'could not receive server id from Broker Dispatcher');
                        serverId = '';
                    }
                    else {
                        logger_1.log.info({ serverId }, 'received server id');
                        clientOpts.config.connections[key].serverId = serverId;
                    }
                }
            }
            else {
                serverId = await (0, dispatcher_1.getServerId)(clientOpts.config, clientOpts.config.brokerToken, brokerClientId);
                if (serverId === null) {
                    logger_1.log.warn({}, 'could not receive server id from Broker Dispatcher');
                    serverId = '';
                }
                else {
                    logger_1.log.info({ serverId }, 'received server id');
                    clientOpts.config.serverId = serverId;
                }
            }
        }
        if ((0, scm_1.commitSigningEnabled)(clientOpts.config)) {
            const commitSigningRules = (0, scm_1.commitSigningFilterRules)();
            if (clientOpts['universalBrokerEnabled']) {
                (_a = clientOpts.filters['github'].private) === null || _a === void 0 ? void 0 : _a.push(...commitSigningRules);
                (_b = clientOpts.filters['github-enterprise'].github.private) === null || _b === void 0 ? void 0 : _b.push(...commitSigningRules);
            }
            else {
                if (clientOpts.filters instanceof Map) {
                    logger_1.log.error({ clientOpts }, 'Error pushing commit signing rules, unexpected filters type');
                    throw new Error('Error pushing commit signing rules, unexpected filters type');
                }
                else {
                    clientOpts.filters.private.push(...commitSigningRules);
                }
            }
            logger_1.log.info({ enabled: true, rulesCount: commitSigningRules.length }, 'loading commit signing rules');
        }
        if (clientOpts.config.INSECURE_DOWNSTREAM) {
            logger_1.log.warn({}, 'Caution! Running in insecure downstream mode, making downstream calls over http, data is not encrypted');
        }
        return {
            preflightCheckResults: preflightCheckResults.length
                ? preflightCheckResults
                : undefined,
        };
    }
    catch (error) {
        logger_1.log.error({ error }, 'Error processing startup hooks');
        throw error;
    }
};
exports.processStartUpHooks = processStartUpHooks;
//# sourceMappingURL=processHooks.js.map