"use strict";
// import '../common/http/patch-https-request-for-proxying';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createWebSocket = exports.createWebSockets = void 0;
const primus_1 = __importDefault(require("primus"));
const logger_1 = require("../logs/logger");
const primus_emitter_1 = __importDefault(require("primus-emitter"));
const reconnectHandler_1 = require("./socketHandlers/reconnectHandler");
const identifyHandler_1 = require("./socketHandlers/identifyHandler");
const errorHandler_1 = require("./socketHandlers/errorHandler");
const openHandler_1 = require("./socketHandlers/openHandler");
const closeHandler_1 = require("./socketHandlers/closeHandler");
const client_1 = require("./types/client");
const requestHandler_1 = require("./socketHandlers/requestHandler");
const chunkHandler_1 = require("./socketHandlers/chunkHandler");
const init_1 = require("./socketHandlers/init");
const integrations_1 = require("./utils/integrations");
const token_1 = require("../common/utils/token");
const oauth_1 = require("./auth/oauth");
const createWebSockets = (clientOpts, globalIdentifyingMetadata) => {
    var _a;
    const integrationsKeys = clientOpts.config.connections
        ? Object.keys(clientOpts.config.connections)
        : [];
    const websocketConnections = [];
    for (let i = 0; i < integrationsKeys.length; i++) {
        const socketIdentifyingMetadata = Object.assign({}, globalIdentifyingMetadata);
        socketIdentifyingMetadata.friendlyName = integrationsKeys[i];
        socketIdentifyingMetadata.identifier =
            clientOpts.config.connections[`${integrationsKeys[i]}`]['identifier'];
        const integrationType = clientOpts.config.connections[`${integrationsKeys[i]}`].type;
        // This type is different for broker, ECR/ACR/DockeHub/etc are all container-registry-agent type for broker
        socketIdentifyingMetadata.supportedIntegrationType =
            (0, integrations_1.translateIntegrationTypeToBrokerIntegrationType)(integrationType);
        socketIdentifyingMetadata.serverId =
            (_a = clientOpts.config.connections[`${integrationsKeys[i]}`].serverId) !== null && _a !== void 0 ? _a : '';
        websocketConnections.push((0, exports.createWebSocket)(clientOpts, socketIdentifyingMetadata, client_1.Role.primary));
        websocketConnections.push((0, exports.createWebSocket)(clientOpts, socketIdentifyingMetadata, client_1.Role.secondary));
    }
    return websocketConnections;
};
exports.createWebSockets = createWebSockets;
const createWebSocket = (clientOpts, originalIdentifyingMetadata, role) => {
    var _a, _b, _c;
    const identifyingMetadata = Object.assign({}, originalIdentifyingMetadata);
    identifyingMetadata.role = role !== null && role !== void 0 ? role : client_1.Role.primary;
    const localClientOps = Object.assign({}, clientOpts);
    identifyingMetadata.identifier =
        (_a = identifyingMetadata.identifier) !== null && _a !== void 0 ? _a : localClientOps.config.brokerToken;
    const Socket = primus_1.default.createSocket({
        transformer: 'engine.io',
        parser: 'EJSON',
        plugin: {
            emitter: primus_emitter_1.default,
        },
        pathname: clientOpts.config.universalBrokerEnabled
            ? `/primus/${identifyingMetadata.identifier}`
            : `/primus/${localClientOps.config.brokerToken}`,
    });
    const urlWithServerIdAndRole = new URL(localClientOps.config.brokerServerUrl);
    const serverId = (_b = localClientOps.config.serverId) !== null && _b !== void 0 ? _b : identifyingMetadata.serverId;
    if (serverId && serverId > -1) {
        urlWithServerIdAndRole.searchParams.append('server_id', serverId);
    }
    urlWithServerIdAndRole.searchParams.append('connection_role', role !== null && role !== void 0 ? role : client_1.Role.primary);
    localClientOps.config.brokerServerUrlForSocket =
        urlWithServerIdAndRole.toString();
    // Will exponentially back-off from 0.5 seconds to a maximum of 20 minutes
    // Retry for a total period of around 4.5 hours
    const socketSettings = {
        reconnect: {
            factor: 1.5,
            retries: 30,
            max: 20 * 60 * 1000,
        },
        ping: parseInt(localClientOps.config.socketPingInterval) || 25000,
        pong: parseInt(localClientOps.config.socketPongTimeout) || 10000,
        timeout: parseInt(localClientOps.config.socketConnectTimeout) || 10000,
    };
    if (clientOpts.accessToken) {
        socketSettings['transport'] = {
            extraHeaders: {
                Authorization: (_c = clientOpts.accessToken) === null || _c === void 0 ? void 0 : _c.authHeader,
            },
        };
    }
    const websocket = new Socket(localClientOps.config.brokerServerUrlForSocket, socketSettings);
    websocket.socketVersion = 1;
    websocket.socketType = 'client';
    if (localClientOps.config.universalBrokerEnabled) {
        websocket.identifier = identifyingMetadata.identifier;
        websocket.supportedIntegrationType =
            identifyingMetadata.supportedIntegrationType || '';
        websocket.serverId = serverId || '';
        websocket.friendlyName = identifyingMetadata.friendlyName || '';
    }
    else {
        websocket.identifier = (0, token_1.maskToken)(identifyingMetadata.identifier);
    }
    websocket.clientConfig = identifyingMetadata.clientConfig;
    websocket.role = identifyingMetadata.role;
    if (clientOpts.accessToken) {
        let timeoutHandlerId;
        let timeoutHandler = async () => { };
        timeoutHandler = async () => {
            logger_1.log.debug({}, 'Refreshing oauth access token');
            clearTimeout(timeoutHandlerId);
            clientOpts.accessToken = await (0, oauth_1.fetchJwt)(clientOpts.config.API_BASE_URL, clientOpts.config.brokerClientConfiguration.common.oauth.clientId, clientOpts.config.brokerClientConfiguration.common.oauth.clientSecret);
            websocket.transport.extraHeaders['Authorization'] =
                clientOpts.accessToken.authHeader;
            websocket.end();
            websocket.open();
            timeoutHandlerId = setTimeout(timeoutHandler, (clientOpts.accessToken.expiresIn - 60) * 1000);
        };
        timeoutHandlerId = setTimeout(timeoutHandler, (clientOpts.accessToken.expiresIn - 60) * 1000);
    }
    websocket.on('incoming::error', (e) => {
        websocket.emit('error', { type: e.type, description: e.description });
    });
    logger_1.log.info({
        url: localClientOps.config.brokerServerUrlForSocket,
        serverId: serverId,
    }, `broker client is connecting to broker server ${role}`);
    (0, init_1.initializeSocketHandlers)(websocket, localClientOps);
    // Websocket events
    websocket.on('identify', (serverData) => (0, identifyHandler_1.identifyHandler)(serverData, websocket));
    websocket.on('reconnect scheduled', reconnectHandler_1.reconnectScheduledHandler);
    websocket.on('reconnect failed', () => (0, reconnectHandler_1.reconnectFailedHandler)(websocket));
    websocket.on('chunk', (0, chunkHandler_1.chunkHandler)(localClientOps.config.universalBrokerEnabled
        ? identifyingMetadata.identifier
        : localClientOps.config.brokerToken));
    // prealably initialized
    websocket.on('request', (0, requestHandler_1.requestHandler)(localClientOps.config.universalBrokerEnabled
        ? identifyingMetadata.identifier
        : localClientOps.config.brokerToken));
    websocket.on('error', errorHandler_1.errorHandler);
    websocket.on('open', () => (0, openHandler_1.openHandler)(websocket, localClientOps, identifyingMetadata));
    websocket.on('close', () => (0, closeHandler_1.closeHandler)(localClientOps, identifyingMetadata));
    // only required if we're manually opening the connection
    // websocket.open();
    return websocket;
};
exports.createWebSocket = createWebSocket;
//# sourceMappingURL=socket.js.map